@@ -227,6 +227,13 @@ def serializeIt(fromJson, toClass, fatherClass=None, muteLogs=False):
         return objectInstance
 
 
+@Function
+def convertFromJsonToDictionary(string):
+    if isinstance(string, str):
+        return json.loads(string)
+    return string
+
+
 @Function
 def convertFromJsonToObject(fromJson, toClass, fatherClass=None, muteLogs=False):
     if ObjectHelper.isNone(fromJson) or ObjectHelper.isNone(toClass):
